Attribute VB_Name = "mBitmap"
Option Explicit

'Types
    Public Type tBitmapInfo
        'Public
        DC As Long
        
        'Internal
        MemoryDC As Boolean
        
        OriginalObject As Long
        PictureDisp As IPictureDisp
        CompatibleBitmap As Long
    End Type
    
    Public Type tBitmap
        'Internal
        FileName As String
        Masked As Boolean
        
        'Bitmap info
        Picture As tBitmapInfo
        Mask As tBitmapInfo
        
        'Size
        W As Long
        H As Long
    End Type

'Declares
    Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal iDC As Long) As Long
    Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal iDC As Long, _
        ByVal iWidth As Long, ByVal iHeight As Long) As Long
    
    Private Declare Function SelectObject Lib "gdi32" (ByVal iDC As Long, ByVal iObject As Long) As Long
    Private Declare Function DeleteObject Lib "gdi32" (ByVal iObject As Long) As Long
    Private Declare Function DeleteDC Lib "gdi32" (ByVal iDC As Long) As Long
    
    Public Declare Function SetPixel Lib "gdi32" (ByVal iDC As Long, _
        ByVal X As Long, ByVal Y As Long, ByVal iColor As Long) As Long
    Public Declare Function GetPixel Lib "gdi32" (ByVal iDC As Long, _
        ByVal X As Long, ByVal Y As Long) As Long
    
    Public Declare Function Rectangle Lib "gdi32" (ByVal iDC As Long, _
        ByVal X1 As Long, ByVal Y1 As Long, _
        ByVal X2 As Long, ByVal Y2 As Long) As Long

'Variables
    Public BitmapCount As Long
    Public Bitmap() As tBitmap
    
    Private MainWindow As Form

Private Sub CreateMask(iBitmap As tBitmap)
    Dim X As Long
    Dim Y As Long
    
    'Create mask
    With iBitmap
        'Create surface
        CreateBitmap .Mask, .W, .H
        
        'Generate mask
        For Y = 0 To .H
            For X = 0 To .W
                SetPixel .Mask.DC, X, Y, IIf(GetPixel(.Picture.DC, X, Y) = 16777215, 0, 16777215)
            Next
        Next
        
        'Setup
        .Masked = True
    End With
End Sub
Public Sub Initialize(iWindow As Form)
    'Variables
    BitmapCount = -1
    
    'Apply main window
    Set MainWindow = iWindow
    MainWindow.ScaleMode = vbPixels
End Sub

Public Sub ReleaseBitmap(iBitmapInfo As tBitmapInfo)
    With iBitmapInfo
        If .MemoryDC Then
            'Release created DC
            SelectObject .DC, .OriginalObject
            DeleteObject .CompatibleBitmap
            DeleteDC .DC
            
        Else
            'Release loaded DC
            SelectObject .DC, .OriginalObject
            DeleteObject .PictureDisp
            DeleteDC .DC
        End If
    End With
End Sub

Public Sub Reset()
    Dim A As Long
    
    'Release all pictures
    For A = 0 To BitmapCount: UnloadBitmap Bitmap(A): Next
    
    'Release memory
    BitmapCount = -1
    Erase Bitmap
End Sub
Public Function LoadBitmap(iFileName As String, iMasked As Boolean) As Long
On Error GoTo 1

    Dim A As Long
    
    'Check if already loaded
    For A = 0 To BitmapCount
        If Bitmap(A).FileName = LCase(iFileName) Then
            'Bitmap already loaded
            LoadBitmap = A

            Exit Function
        End If
        
        'Use this slot if its empty
        If Len(Bitmap(A).FileName) = 0 Then: Exit For
    Next
    
    'Check filename
    If Dir(iFileName, vbNormal) = "" Or Len(iFileName) = 0 Then
        'Return -1 to show an error
        LoadBitmap = -1
        
        Exit Function
    End If
    
    'If no empty bitmap found
    If A > BitmapCount Then
        'Allocate memory for the new bitmap
        BitmapCount = BitmapCount + 1
        ReDim Preserve Bitmap(BitmapCount)
    End If
    
    With Bitmap(A)
        With .Picture
            'Setup bitmap
            .MemoryDC = False
            
            'Load picture and set object
            .DC = CreateCompatibleDC(MainWindow.hdc)
            Set .PictureDisp = LoadPicture(iFileName)
            .OriginalObject = SelectObject(.DC, .PictureDisp)
        End With
        
        'Internal
        .FileName = LCase(iFileName)
        
        'Get size in pixel
        .W = MainWindow.ScaleX(.Picture.PictureDisp.Width)
        .H = MainWindow.ScaleY(.Picture.PictureDisp.Height)
    End With
    
    'Load mask
    If iMasked Then: CreateMask Bitmap(A)
    
    'Return index
    LoadBitmap = BitmapCount
    Exit Function
    
1  'Error
    LoadBitmap = -1
End Function
Public Function CreateBitmap(iBitmapInfo As tBitmapInfo, iW As Long, iH As Long) As Boolean
On Error GoTo 1

    With iBitmapInfo
        'Set info
        .MemoryDC = True
        
        'Create empty surface
        .DC = CreateCompatibleDC(MainWindow.hdc)
        .CompatibleBitmap = CreateCompatibleBitmap(MainWindow.hdc, iW, iH)
        .OriginalObject = SelectObject(.DC, .CompatibleBitmap)
    End With
    
    'Return true
    CreateBitmap = True
    Exit Function
    
1  'Error
    ReleaseBitmap iBitmapInfo
    
    'Return error
    CreateBitmap = False
End Function

Public Sub UnloadBitmap(iBitmap As tBitmap)
    With iBitmap
        'Release picture
        ReleaseBitmap .Picture
        
        'Internal
        .FileName = ""
    End With
End Sub

