Attribute VB_Name = "mMap"
Option Explicit

'Types
    Public Type tMap
        W As Long 'W stands for width
        H As Long 'H stands for height
        
        'Maps
        Floor() As Long
        Roof() As Long
        Units() As Long
    End Type

'Variables
    Public Map As tMap
Public Sub CreateMap(iW As Long, iH As Long)
    Dim A As Long
    
    With Map
        'Get size
        .W = iW
        .H = iH
        
        'Create maps
        ReDim .Floor(.W * .H)
        ReDim .Roof(.W * .H)
        ReDim .Units(.W * .H)
    End With
End Sub

Public Sub Reset()
    'Reset map
    Map.W = 0
    Map.H = 0
    
    Erase Map.Floor
    Erase Map.Roof
End Sub

Public Sub DrawFloor()
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    Dim TileIndex As Long
    
    Dim TempX As Single
    Dim TempY As Single
    
    With Map
        'Get map position
        TempX = (Camera.X / TileSize)
        TempY = (Camera.Y / TileSize)
        
        'Calculate index of this positon
        Index = Int(TempY) * .W + Int(TempX)
        
        'Calculate offset
        TempX = TempX - Int(TempX)
        TempY = TempY - Int(TempY)
        
        'Draw the map
        For Y = 0 To Camera.TilesY
            For X = 0 To Camera.TilesX
                'Check index
                If Index > (Map.W * .H) Then: Exit Sub
                
                'Get tile index
                TileIndex = .Floor(Index)
                
                'Draw tile
                BitBlt Camera.FrontDC, _
                    (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                    TileSize, TileSize, _
                    Bitmap(Tile(TileIndex).Bitmap).Picture.DC, _
                    Tile(TileIndex).ActFrame * TileSize, 0, _
                    vbSrcCopy
                
                'Next tile
                Index = Index + 1
            Next
            
            'Get next line start
            Index = Index + .W - Camera.TilesX - 1
        Next
    End With
End Sub
Public Sub DrawUnits()
    Dim A As Long
    Dim Done As Boolean
    
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    
    Dim UnitListCount As Long
    Dim UnitList() As Long
    
    'Calculate start index
    Index = Int((Camera.Y / TileSize)) * Map.W + Int((Camera.X / TileSize))
    UnitListCount = -1
    
    'Draw the map
    For Y = 0 To Camera.TilesY
        For X = 0 To Camera.TilesX
            'Check index
            If Index > (Map.W * Map.H) Then: Exit For
            
            'Get unit index
            If Map.Units(Index) > 0 Then
                'Allocate memory
                UnitListCount = UnitListCount + 1
                ReDim Preserve UnitList(UnitListCount)
                
                'Add unit index
                UnitList(UnitListCount) = Map.Units(Index)
            End If
            
            'Next tile
            Index = Index + 1
        Next
        
        'Get next line start
        Index = Index + Map.W - Camera.TilesX - 1
    Next
    
    'Z-sort the list
    While Not Done
        Done = True
        
        For A = 0 To UnitListCount - 1
            'If unit is in front of next unit
            If Unit(UnitList(A)).Y > Unit(UnitList(A + 1)).Y Then
                'Swap items
                Index = UnitList(A)
                UnitList(A) = UnitList(A + 1)
                UnitList(A + 1) = Index
                
                'Check again
                Done = False
            End If
        Next
    Wend
    
    'Now draw the sorted list
    For A = 0 To UnitListCount
        DrawUnit UnitList(A)
    Next
End Sub
Public Sub DrawRoof()
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    Dim TileIndex As Long
    
    Dim TempX As Single
    Dim TempY As Single
    
    With Map
        'Get map position
        TempX = (Camera.X / TileSize)
        TempY = (Camera.Y / TileSize)
        
        'Calculate index of this positon
        Index = Int(TempY) * .W + Int(TempX)
        
        'Calculate offset
        TempX = TempX - Int(TempX)
        TempY = TempY - Int(TempY)
        
        'Draw the map
        For Y = 0 To Camera.TilesY
            For X = 0 To Camera.TilesX
                'Check index
                If Index > (.W * .H) Then: Exit Sub
                
                'Get tile index
                TileIndex = .Roof(Index)
                
                If TileIndex > 0 Then
                    'Draw tile mask
                    BitBlt Camera.FrontDC, _
                        (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                        TileSize, TileSize, _
                        Bitmap(Tile(TileIndex).Bitmap).Mask.DC, _
                        Tile(TileIndex).ActFrame * TileSize, 0, _
                        vbSrcPaint 'logical AND
                    
                    'Draw roof tile
                    BitBlt Camera.FrontDC, _
                        (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                        TileSize, TileSize, _
                        Bitmap(Tile(TileIndex).Bitmap).Picture.DC, _
                        Tile(TileIndex).ActFrame * TileSize, 0, _
                        vbSrcAnd 'logical OR
                End If
                
                'Next tile
                Index = Index + 1
            Next
            
            'Get next line start
            Index = Index + .W - Camera.TilesX - 1
        Next
    End With
End Sub

Public Function SaveData(iFileName As String) As Boolean
On Error GoTo 1

    Dim FileNumber As Integer
    
    'Open the specified file
    FileNumber = FreeFile
    Open iFileName For Binary As FileNumber
        'Write data
        WriteData FileNumber
    Close FileNumber
    
    'We're done
    SaveData = True
    Exit Function

1  'Catched an error
    SaveData = False
End Function

Public Function LoadData(iFileName As String) As Boolean
On Error GoTo 1

    Dim FileNumber As Integer
    
    'Check existance
    If Dir(iFileName, vbNormal) = "" Then: GoTo 1
    
    'Open the specified file
    FileNumber = FreeFile
    Open iFileName For Binary As FileNumber
        'Read data
        ReadData FileNumber
    Close FileNumber
    
    'We're done
    LoadData = True
    Exit Function

1  'Catched an error
    LoadData = False
End Function

Public Sub SetFloorTile(iX As Long, iY As Long, Index As Long)
    'Set tile at the specified 0-based position
    Map.Floor(iY * Map.W + iX) = Index
End Sub

Public Sub SetRoofTile(iX As Long, iY As Long, Index As Long)
    'Set tile at the specified 0-based position
    Map.Roof(iY * Map.W + iX) = Index
End Sub
Public Sub WriteData(iFileNumber As Integer)
    'Write tile set
    mTiles.WriteData iFileNumber
    
    With Map
        'Write map size
        Put iFileNumber, , .W
        Put iFileNumber, , .H
        
        'Write map data
        Put iFileNumber, , .Floor
        Put iFileNumber, , .Roof
    End With
End Sub

Public Sub ReadData(iFileNumber As Integer)
    'Read tile set
    mTiles.ReadData iFileNumber
        
    With Map
        'Read m size
        Get iFileNumber, , .W
        Get iFileNumber, , .H
        
        'Allocate memory
        CreateMap .W, .H
        
        'Read map data
        Get iFileNumber, , .Floor
        Get iFileNumber, , .Roof
    End With
End Sub



