Attribute VB_Name = "mPathFinding"
Option Explicit

'Types
    Public Type tCoord2D
        X As Long
        Y As Long
    End Type

'Variables
    Private OpenList As Collection
    Private DoneList As Collection
    
    Private TargetUnit As Long
    
    Private Destination As tCoord2D

Public Sub FindPath(iTargetUnit As Long, iDestination As tCoord2D)
    Dim Temp As New cNode
    
    'Setup target
    TargetUnit = iTargetUnit
    Destination = iDestination
    
    'Setup unit
    Unit(TargetUnit).WayPointCount = -1
    
    'Reset lists
    Set OpenList = New Collection
    Set DoneList = New Collection
    
    'Setup start
    Temp.X = CLng(Unit(TargetUnit).X / TileSize)
    Temp.Y = CLng(Unit(TargetUnit).Y / TileSize)
    
    'Start path finding
    DoneList.Add Temp
    CheckNode Temp
End Sub

Private Sub AddOpen(X As Long, Y As Long, Score As Long, iParent As cNode)
    Dim TempNode As cNode
    Dim TempScore As Long
    
    'Check position
    If X < 0 Then: Exit Sub
    If Y < 0 Then: Exit Sub
    If X > (Map.W - 1) Then: Exit Sub
    If Y > (Map.H - 1) Then: Exit Sub
    
    'Check unit collisions
    If Map.Units(Y * Map.W + X) > 0 Then: Exit Sub
    
    'Check map friction (Score)
    TempScore = Tile(Map.Floor(Y * Map.W + X)).Friction
    If TempScore < 0 Then: Exit Sub
    
    'Add Score offset
    TempScore = TempScore + Score
    
    'Check done list
    For Each TempNode In DoneList
        If TempNode.X = X And TempNode.Y = Y Then: Exit Sub
    Next
    
    'Check open list
    For Each TempNode In OpenList
        If (TempNode.X = X) And (TempNode.Y = Y) Then
            'Check new path
            If TempNode.Score >= TempScore Then
                'Found a nearer path
                Set TempNode.Parent = iParent
                TempNode.Score = TempScore
            End If
            
            Exit Sub
        End If
    Next
    
    'Create node
    Set TempNode = New cNode
    OpenList.Add TempNode
    
    'Setup node
    With TempNode
        .X = X
        .Y = Y
        .Dist = (Abs(Destination.X - X) + Abs(Destination.Y - Y)) * 10
'        .Dist = Sqr((Destination.X - X) ^ 2 + (Destination.Y - Y) ^ 2) * 10
        .Score = TempScore
        Set .Parent = iParent
    End With
End Sub

Private Sub CheckBack(iNode As cNode)
    'Add waypoint
    With Unit(TargetUnit)
        'Create waypoint
        .WayPointCount = .WayPointCount + 1
        ReDim Preserve .WayPoint(.WayPointCount)
        
        'Setup waypoint
        .WayPoint(.WayPointCount).X = iNode.X
        .WayPoint(.WayPointCount).Y = iNode.Y
    End With
    
    If Not iNode.Parent Is Nothing Then
        'Check next parent
        CheckBack iNode.Parent
    End If
End Sub

Private Sub CheckNextItem()
    Dim A As Long
    
    Dim Min As Long
    Dim Index As Long
    
    Dim TempNode As cNode
    
    'Check open list
    If OpenList.Count = 0 Then: Exit Sub
    
    'Setup
    Index = 1
    Set TempNode = OpenList.Item(1)
    Min = TempNode.Dist + TempNode.Score
    
    'Find nearest itemNode.Dist
    For A = 2 To OpenList.Count
        'Get node from collection
        Set TempNode = OpenList.Item(A)
        
        'Compare distance
        If (TempNode.Dist + TempNode.Score) <= Min Then
            Index = A
            Min = TempNode.Dist
        End If
    Next
    
    'Get nearest item
    Set TempNode = OpenList(Index)
    
    'Move to done list
    DoneList.Add TempNode
    OpenList.Remove Index
    
    'Check that item
    CheckNode TempNode
End Sub

Private Sub CheckNode(iNode As cNode)
    Dim X As Long
    Dim Y As Long
    Dim Score As Long
    Dim Index As Long
    
    'Get values
    X = iNode.X
    Y = iNode.Y
    Score = iNode.Score
    
    'Check position
    If X = Destination.X And Y = Destination.Y Then
        'Done
        CheckBack iNode
        
        Exit Sub
    End If
    
    'Add straight nodes
    AddOpen X + 1, Y, Score + 10, iNode
    AddOpen X, Y + 1, Score + 10, iNode
    AddOpen X - 1, Y, Score + 10, iNode
    AddOpen X, Y - 1, Score + 10, iNode
    
'    'Add diagonal nodes
'    AddOpen X + 1, Y + 1, Score + 14, iNode
'    AddOpen X + 1, Y - 1, Score + 14, iNode
'    AddOpen X - 1, Y + 1, Score + 14, iNode
'    AddOpen X - 1, Y - 1, Score + 14, iNode
    
    'Check list
    CheckNextItem
End Sub


