Attribute VB_Name = "mTiles"
Option Explicit

'Types
    Public Type tTile
        'Tile info
        PictureFile As String
        Bitmap As Long
        
        'Animation setup
        FrameCount As Integer
        FrameDelay As Long
        
        'Animation data
        ActFrame As Integer
        NextTick As Long
        
        'Game properties
        Friction As Long
    End Type

'Variables
    Public TileCount As Long
    Public Tile() As tTile
    
    Public TileSize As Long

Public Sub LoadResources()
    Dim A As Long
    
    For A = 1 To TileCount
        'Load the bitmap and get it's index
        Tile(A).Bitmap = mBitmap.LoadBitmap(SourcePath & Tile(A).PictureFile, True)
    Next
End Sub

Public Sub WriteData(iFileNumber As Integer)
    Dim A As Long
    
    'Write array size
    Put iFileNumber, , TileCount
    
    For A = 0 To TileCount
        With Tile(A)
            'Put the file name
            WriteString iFileNumber, .PictureFile
            
            'Animation info
            Put iFileNumber, , .FrameCount
            Put iFileNumber, , .FrameDelay
            
            'Game properties
            Put iFileNumber, , .Friction
        End With
    Next
End Sub
Public Sub ReadData(iFileNumber As Integer)
    Dim A As Long
    
    'Read array size and allocate memory
    Get iFileNumber, , TileCount
    ReDim Tile(TileCount)
    
    For A = 0 To TileCount
        With Tile(A)
            'Get the file name
            ReadString iFileNumber, .PictureFile
            
            'Animation info
            Get iFileNumber, , .FrameCount
            Get iFileNumber, , .FrameDelay
            
            'Game properties
            Get iFileNumber, , .Friction
        End With
    Next
End Sub

Public Sub AddAnimTile(iPictureFile As String, iFrameCount As Integer, iFrameDelay As Long, iWalkable As Boolean)
    'Allocate memory
    TileCount = TileCount + 1
    ReDim Preserve Tile(TileCount)
    
    With Tile(TileCount)
        'Setup tile
        .PictureFile = iPictureFile
        
        'Setup animation info
        .FrameCount = iFrameCount
        .FrameDelay = iFrameDelay
        
        'Reset animation
        .ActFrame = 0
        .NextTick = 0
        
        'Setup game properties
        .Walkable = iWalkable
    End With
End Sub

Public Sub AddTile(iPictureFile As String, iFriction As Long)
    'Allocate memory
    TileCount = TileCount + 1
    ReDim Preserve Tile(TileCount)
    
    With Tile(TileCount)
        'Setup tile
        .PictureFile = iPictureFile
        
        'Setup animation
        .FrameCount = 0
        .FrameDelay = 0
        
        'Reset animation
        .ActFrame = 0
        .NextTick = 0
        
        'Setup game properties
        .Friction = iFriction
    End With
End Sub

Public Sub Animate()
    Dim A As Long
    Dim Temp As Long
    
    'Get current tick
    Temp = GetTickCount
    
    'Check all tiles
    For A = 0 To TileCount
        With Tile(A)
            'Check timing
            If .NextTick < Temp Then
                'Increase frame number
                .ActFrame = .ActFrame + 1
                
                'Reset animation if last frame reached
                If .ActFrame > .FrameCount Then: .ActFrame = 0
                
                'Set next tick
                .NextTick = Temp + .FrameDelay
            End If
        End With
    Next
End Sub

Public Sub Reset()
    'Reset tiles
    TileCount = -1
    Erase Tile
    
    'Further setup
    TileSize = 16
End Sub
