Attribute VB_Name = "mUnits"
Option Explicit

'Types
    Public Type tUnit
        'Movement (in pixel)
        X As Long
        Y As Long
        
        Speed As Single
        
        'Path finding
        WayPointCount As Long
        WayPoint() As tCoord2D
        
        'Current map position
        MapIndex As Long
        Moved As Boolean
        
        'Unit size
        W As Long
        H As Long
        
        'Animation setup
        FrameCount As Integer
        FrameDelay As Long
        
        'Animation data
        Animated As Boolean
        ActFrame As Integer
        NextTick As Long
        
        'Display
        Bitmap As Long
        Direction As Long
    End Type

'Variables
    Public UnitCount As Long
    Public Unit() As tUnit

Public Sub AddUnit(iPictureFile As String, iW As Long, iH As Long, iFrameCount As Long, iFrameDelay As Long, iSpeed As Single)
    'Allocate memory
    UnitCount = UnitCount + 1
    ReDim Preserve Unit(UnitCount)
    
    With Unit(UnitCount)
        'Load bitmap
        .Bitmap = LoadBitmap(SourcePath & iPictureFile, True)
        
        'Setup size
        .W = iW
        .H = iH
        
        'Setup animation info
        .FrameCount = iFrameCount
        .FrameDelay = iFrameDelay
        
        'Reset animation
        .ActFrame = 0
        .NextTick = 0
        
        'Setup path finding
        .Speed = iSpeed
        .WayPointCount = -1
        
        'Reset map index
        .MapIndex = -1
    End With
End Sub

Public Sub Animate()
    Dim A As Long
    Dim Temp As Long
    
    'Get current tick
    Temp = GetTickCount
    
    'Check all units
    For A = 0 To UnitCount
        With Unit(A)
            'Check timing
            If .NextTick < Temp Then
                'Check if animated
                If .Animated Then
                    'Increase frame number
                    .ActFrame = .ActFrame + 1
                    
                    'Reset animation if last frame reached
                    If .ActFrame > .FrameCount Then: .ActFrame = 0
                    
                    'Set next tick
                    .NextTick = Temp + .FrameDelay
                
                Else
                    'Reset animation
                    .ActFrame = 0
                End If
            End If
        End With
    Next
End Sub
Public Sub DrawUnit(Index As Long)
    Dim OffsetX As Long
    Dim OffsetY As Long
    
    Dim DrawX As Long
    Dim DrawY As Long
    
    With Unit(Index)
        'Get offset position (bottom-center)
        OffsetX = .W / 2 - TileSize / 2
        OffsetY = .H - TileSize / 2
        
        'Get final draw position
        DrawX = (.X - OffsetX) - Camera.X
        DrawY = (.Y - OffsetY) - Camera.Y
        
        'Draw mask
        BitBlt Camera.FrontDC, _
            DrawX, DrawY, _
            .W, .H, _
            Bitmap(.Bitmap).Mask.DC, _
            (.W * .ActFrame), (.H * .Direction), _
            vbSrcPaint
        
        'Draw picture
        BitBlt Camera.FrontDC, _
            DrawX, DrawY, _
            .W, .H, _
            Bitmap(.Bitmap).Picture.DC, _
            (.W * .ActFrame), (.H * .Direction), _
            vbSrcAnd
        
        Dim A As Long
        
        'Show waypoints
        For A = 0 To .WayPointCount
            With .WayPoint(A)
                Rectangle Camera.FrontDC, (.X * TileSize) - Camera.X, (.Y * TileSize) - Camera.Y, ((.X + 1) * TileSize) - Camera.X, ((.Y + 1) * TileSize) - Camera.Y
            End With
        Next
    End With
End Sub

Public Sub MoveRandomly()
    Dim A As Long
    Dim Destination As tCoord2D
    
    For A = 2 To UnitCount
        If Unit(A).WayPointCount < 0 Then
            With Destination
                Do
                    'Get random position
                    .X = (Map.W - 1) * Rnd
                    .Y = (Map.H - 1) * Rnd
                Loop While Tile(Map.Floor(.Y * Map.W + .X)).Friction < 0
            End With
        End If
        
        'Get path
        FindPath A, Destination
    Next
End Sub

Public Sub MoveUnit(Index As Long, NewX As Long, NewY As Long)
    Dim TempIndex As Long
    
    'Check boundaries
    If NewX < 0 Then: NewX = 0
    If NewY < 0 Then: NewY = 0
    If NewX > ((Map.W - 1) * TileSize) Then: NewX = ((Map.W - 1) * TileSize)
    If NewY > ((Map.H - 1) * TileSize) Then: NewY = ((Map.H - 1) * TileSize)
    
    'Get new map index
    TempIndex = (CLng(NewY / TileSize) * Map.W) + CLng(NewX / TileSize)
    
    With Unit(Index)
        'Reset movement indicator
        .Moved = False
        
        If Not TempIndex = .MapIndex Then
            'Unit-to-map collision check
            If Tile(Map.Floor(TempIndex)).Friction < 0 Then: Exit Sub
            
            'Unit-to-unit collision check
            If Map.Units(TempIndex) > 0 Then: Exit Sub
        End If
        
        'Unregister from map
        If .MapIndex > -1 Then: Map.Units(.MapIndex) = 0
        
        'Apply new position
        .X = NewX
        .Y = NewY
        .MapIndex = TempIndex
        
        'We moved
        .Moved = True
        
        'Register to map
        Map.Units(.MapIndex) = Index
    End With
End Sub

Public Sub Reset()
    'Reset units
    UnitCount = -1
    Erase Unit
End Sub

Public Sub UpdateUnits()
    Dim A As Long
    Dim Angle As Single
    
    Dim TempDestination As tCoord2D
    
    'Frequently update
    For A = 1 To UnitCount
        With Unit(A)
            If .WayPointCount > -1 Then
                'Get waypoint position
                TempDestination.X = .WayPoint(.WayPointCount).X * TileSize
                TempDestination.Y = .WayPoint(.WayPointCount).Y * TileSize
                
                'Check distance
                If Distance(.X, .Y, TempDestination.X, TempDestination.Y) < (TileSize / 4) Then
                    'Step to next waypoint
                    .WayPointCount = .WayPointCount - 1
                Else
                    'Get angle to next waypoint
                    Angle = GetAngle(CSng(.X), CSng(.Y), CSng(TempDestination.X), CSng(TempDestination.Y))
                    
                    'Get 4-based direction
                    If Angle >= (Pi / 4) * 5 And Angle < (Pi / 4) * 7 Then: .Direction = 0
                    If Angle >= (Pi / 4) * 7 Or Angle < (Pi / 4) * 1 Then: .Direction = 1
                    If Angle >= (Pi / 4) * 1 And Angle < (Pi / 4) * 3 Then: .Direction = 2
                    If Angle >= (Pi / 4) * 3 And Angle < (Pi / 4) * 5 Then: .Direction = 3
                    
                    'Move in this direction
                    MoveUnit A, .X + (Cos(Angle) * .Speed), .Y + (Sin(Angle) * .Speed)
                End If
            End If
            
            'Animate if moved
            .Animated = .Moved
            .Moved = False
        End With
    Next
End Sub

